/***************************************************************************
 * qlocx_cellularcon_obj.c 				                                                
 *											
 *  
 *  
 *                                                                                      
 * Functional level: 1									
 *											
 * Prepared: Dirk Handzic							
 *																					
 * 											
 * Rev	Date	Author	Ext int. (Y/N) 	Reason/description
 * 0.1  210118  DH	Y        	Initial version
 *
 *  
 * Cellular Connectivity LWM2M configuration 										
 ***************************************************************************/
#include        <zephyr.h>
#include        <stdlib.h>
#include        <nrf9160.h>
#include        <drivers/spi.h>
#include        <drivers/gpio.h>
#include        <zephyr.h>
#include        <net/lwm2m.h>
#include        <stdint.h>
#include        <string.h>
#include         <stdio.h>
#include        <init.h>
#include	"qlocx_digio.h"
#include        <logging/log.h>
#include        "lwm2m_object.h"
#include        "lwm2m_engine.h"

//*************************** Definitions ***********************************

#define LOG_MODULE_NAME net_lwm2m_obj_qlocx_cellular
#define LOG_LEVEL CONFIG_LWM2M_LOG_LEVEL

LOG_MODULE_REGISTER(LOG_MODULE_NAME);

#define LWM2M_OBJECT_CELLULAR_ID                10

/* resource IDs */
#define PSM_TIMER_ID                            4
#define ACTIVE_TIMER_ID                         5
#define EDRX_WBS1_ID                            8
#define ACTIVATED_PROF_NAME_ID                  11

#define CELLULAR_MAX_ID				4

#define ACTIVATED_PROF_NAME_MAX                 4

/*
 * Calculate resource instances as follows:
 * start with CELLULAR_MAX_ID
 * add ACTIVATED_PROF_NAME_MAX for ACTIVATED_PROF_NAME  resource instances
 */
#define RESOURCE_INSTANCE_COUNT	(CELLULAR_MAX_ID + ACTIVATED_PROF_NAME_MAX)

/* resource state variables */
static  uint16_t psm_timer;
static  uint16_t active_timer;
static  uint8_t edrx_value;


static struct lwm2m_engine_obj cellular;
static struct lwm2m_engine_obj_field fields[] = {
	OBJ_FIELD_DATA(PSM_TIMER_ID, RW, U16),
	OBJ_FIELD_DATA(ACTIVE_TIMER_ID, RW, U16),
	OBJ_FIELD_DATA(EDRX_WBS1_ID, RW, U8),
        OBJ_FIELD_DATA(ACTIVATED_PROF_NAME_ID, R, OBJLNK),        
};

static struct lwm2m_engine_obj_inst inst;
static struct lwm2m_engine_res res[CELLULAR_MAX_ID];
static struct lwm2m_engine_res_inst res_inst[RESOURCE_INSTANCE_COUNT];

static struct lwm2m_engine_obj_inst *cellular_create(uint16_t obj_inst_id)
{
uint8_t edrx;
uint8_t ptw;
int i = 0, j = 0;

	init_res_instance(res_inst, ARRAY_SIZE(res_inst));

	// Set default values
	psm_timer = 14400;
	active_timer = 16;
        edrx = strtoul(CONFIG_LTE_EDRX_REQ_VALUE, NULL, 2);
        ptw = strtoul(CONFIG_LTE_PTW_VALUE, NULL, 2);
	edrx_value = (edrx << 4) | ptw;

	// initialize instance resource data 
	INIT_OBJ_RES_DATA(PSM_TIMER_ID, res, i, res_inst, j,  &psm_timer, sizeof(psm_timer));
	
	INIT_OBJ_RES_DATA(ACTIVE_TIMER_ID, res, i, res_inst, j, &psm_timer, sizeof(active_timer));

	INIT_OBJ_RES_DATA(EDRX_WBS1_ID, res, i, res_inst, j, &edrx_value, sizeof(edrx_value));

        INIT_OBJ_RES_DATA(ACTIVATED_PROF_NAME_ID, res, i, res_inst, j, ACTIVATED_PROF_NAME_MAX, false);
	

	inst.resources = res;
	inst.resource_count = i;

	LOG_DBG("Create IPSO Cellular Connectivity instance: %d", obj_inst_id);

	return &inst;
}

static int ipso_cellular_init(const struct device *dev)
{	
int ret = 0;
struct lwm2m_engine_obj_inst *obj_inst = NULL;

	cellular.obj_id = LWM2M_OBJECT_CELLULAR_ID;
	cellular.fields = fields;
	cellular.field_count = ARRAY_SIZE(fields);
	cellular.max_instance_count = 1;
	cellular.create_cb = cellular_create;
	lwm2m_register_obj(&cellular);

        // auto create the only instance 
	ret = lwm2m_create_obj_inst(LWM2M_OBJECT_CELLULAR_ID, 0, &obj_inst);
	if (ret < 0) {
		LOG_DBG("Create LWM2M instance 0 error: %d", ret);
	}

	return ret;

}

SYS_INIT(ipso_cellular_init, APPLICATION, CONFIG_KERNEL_INIT_PRIORITY_DEFAULT);