/***************************************************************************
 * qlocx_digio.c 				                                                
 *											
 *  
 *  
 *                                                                                      
 * Functional level: 1									
 *											
 * Prepared: Dirk Handzic							
 *																					
 * 											
 * Rev	Date	Author	Ext int. (Y/N) 	Reason/description
 * 0.2  210112  DH      Y               qlocx_digio_InpPwrOn added   
 * 0.1  201028  DH	Y        	Initial version
 *
 *  
 * Digital I/O handler 										
 ***************************************************************************/

#include        <nrf9160.h>
#include        <drivers/spi.h>
#include        <drivers/gpio.h>
#include        <zephyr.h>
#include	"qlocx_digio.h"
#include        <logging/log.h>

//*************************** Definitions ***********************************

LOG_MODULE_REGISTER(app_qlocx_digio, CONFIG_APP_LOG_LEVEL);

#define CHECK_DELAY     2              // Check every 10 ms
#define TURN_OFF_DELAY  5
//#define AUTO_RESET

#define OLD_ON_OUT1     0x01
#define OLD_ON_OUT2     0x02
#define OLD_ON_OUT3     0x04
#define OLD_ON_OUT4     0x08
#define OLD_ON_OUT5     0x10
#define OLD_ON_OUT6     0x20
#define OLD_ON_OUT7     0x40
#define OLD_ON_OUT8     0x80

#define OPEN_LOAD_DETECT   (OLD_ON_OUT1 | OLD_ON_OUT2 | OLD_ON_OUT3 | OLD_ON_OUT4 | OLD_ON_OUT5 | OLD_ON_OUT6 | OLD_ON_OUT7 | OLD_ON_OUT8 )

static struct device * gpio_dev;
static struct device * spi_dev;

static uint8_t last_output_pwr1 = 0x00;
static uint8_t last_output_pwr2 = 0x00;
static uint8_t last_status_pwr1 = 0x00;
static uint8_t last_status_pwr2 = 0x00;
static uint8_t open_fault_pwr1;
static uint8_t open_fault_pwr2;
static uint8_t short_fault_pwr1;
static uint8_t short_fault_pwr2;

const uint8_t remap_out[16] = {5, 0, 1, 7, 4, 2, 3, 6, 
                               15, 9, 8, 13, 14, 11, 10, 12};
                                
static bool out_state[MAX_OUTPUT];


//*************************** Internal functions ****************************
static struct spi_cs_control cs_pwr1 = {
        .delay = 1,
        .gpio_dev = NULL,
        .gpio_pin = 25,
        };


static struct spi_cs_control cs_pwr2 = {
        .delay = 1,
        .gpio_dev = NULL,
        .gpio_pin = 18,
        };

static struct spi_config spi_cfg = {
	.operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
		     SPI_MODE_CPHA,
	.frequency = 4000000,
	.slave = 0,
        //.cs = &cs_pwr1,
};

static struct spi_config spi_cfg_pwr2 = {
	.operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
		     SPI_MODE_CPHA,
	.frequency = 4000000,
	.slave = 0,
        .cs = &cs_pwr2,
};



static void spi_init(void)
{
char* spiName = "SPI_3";

      spi_dev = device_get_binding(spiName);

      if (spi_dev == NULL) {
              printk("Could not get %s device\n", spiName);
              return;
      }
}


// Set output
// ch = 0 - 15; state = true or false
void    spi_Out(uint8_t ch, uint8_t state)
{
uint8_t output;
uint8_t data;
int err;
static uint8_t tx_buffer[2];
static uint8_t rx_buffer[2];

const struct spi_buf tx_buf = {
        .buf = tx_buffer,
        .len = sizeof(tx_buffer)
};
const struct spi_buf_set tx = {
        .buffers = &tx_buf,
        .count = 1
};

struct spi_buf rx_buf = {
        .buf = rx_buffer,
        .len = sizeof(rx_buffer),
};
const struct spi_buf_set rx = {
        .buffers = &rx_buf,
        .count = 1
};
   
   
  
    if( ch < OPEN_DOOR9){
        gpio_pin_set(gpio_dev, 18, 0);        // CS_PWR2  passive
        gpio_pin_set(gpio_dev, 25, 1);        // CS_PWR1  active
        data = 1 << ch;
        output = last_output_pwr1;
        if(state){        
            output |= data;        
        }
        else{
            output &= ~data;
        }
        
        tx_buffer[1] = output;
        tx_buffer[0] =  OPEN_LOAD_DETECT;
        err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
        last_status_pwr1 = rx_buffer[1];
        open_fault_pwr1 = ~last_output_pwr1 & last_status_pwr1;
        if(open_fault_pwr1 != 0){
            //LOG_ERR("Open outputs 1-8: %d",open_fault_pwr1);
        }
        short_fault_pwr1 = last_output_pwr1 & last_status_pwr1;
        if(short_fault_pwr1 != 0){
            //LOG_ERR("Short outputs 1-8: %d",short_fault_pwr1);
        }
        last_output_pwr1 = output;
    }
    else{
        gpio_pin_set(gpio_dev, 25, 0);        // CS_PWR1  passive
        gpio_pin_set(gpio_dev, 18, 1);        // CS_PWR2  active 
        ch -= 8;    
        data = 1 << ch;
        output = last_output_pwr2;
        if(state){        
            output |= data;        
        }
        else{
            output &= ~data;
        }
        
        tx_buffer[1] = output;
        tx_buffer[0] =  OPEN_LOAD_DETECT;
        err = spi_transceive(spi_dev, &spi_cfg, &tx, &rx);
        last_status_pwr2 = rx_buffer[1];
        open_fault_pwr2 = ~last_output_pwr2 & last_status_pwr2;
        if(open_fault_pwr2 != 0){
            //LOG_ERR("Open outputs 9-16: %d",open_fault_pwr2);
        }
        short_fault_pwr2 = last_output_pwr2 & last_status_pwr2;
        if(short_fault_pwr2 != 0){
            //LOG_ERR("Short outputs 9-16: %d", short_fault_pwr2);
        }
        last_output_pwr2 = output;   
    }
    
    gpio_pin_set(gpio_dev, 25, 0);        // CS_PWR1  passive
    gpio_pin_set(gpio_dev, 18, 0);        // CS_PWR2  passive
    
}

uint8_t digio_RemapOut(uint8_t in)
{
    return( remap_out[in]);
}
//*************************** External functions ****************************
void	digio_Init(void)
{
uint8_t i;

    spi_init();
    gpio_dev = device_get_binding("GPIO_0");
    if (!gpio_dev) {
        LOG_ERR("Could not get GPIO_0 device.");
    }
    cs_pwr1.gpio_dev = gpio_dev;
    cs_pwr2.gpio_dev = gpio_dev;
    if(gpio_pin_configure(gpio_dev, 0, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.0.");
    }
    if(gpio_pin_configure(gpio_dev, 1, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.1.");
    }
    if(gpio_pin_configure(gpio_dev, 2, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.2.");
    }
    if(gpio_pin_configure(gpio_dev, 3, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.3.");
    }
    if(gpio_pin_configure(gpio_dev, 4, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.4.");
    }
    if(gpio_pin_configure(gpio_dev, 5, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.5.");
    }
    if(gpio_pin_configure(gpio_dev, 6, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.6.");
    }
    if(gpio_pin_configure(gpio_dev, 7, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.7.");
    }
    if(gpio_pin_configure(gpio_dev, 8, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.8.");
    }
    if(gpio_pin_configure(gpio_dev, 9, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.9.");
    }
    if(gpio_pin_configure(gpio_dev, 10, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.10.");
    }
    if(gpio_pin_configure(gpio_dev, 11, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.11.");
    }
    if(gpio_pin_configure(gpio_dev, 12, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.12.");
    }
    if(gpio_pin_configure(gpio_dev, 13, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.13.");
    }
    if(gpio_pin_configure(gpio_dev, 14, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.14.");
    }
    if(gpio_pin_configure(gpio_dev, 15, GPIO_INPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.15.");
    }
    if(gpio_pin_configure(gpio_dev, 18, GPIO_OUTPUT | GPIO_ACTIVE_LOW) < 0){
        LOG_ERR("Could not configure P0.18.");
    }
    if(gpio_pin_configure(gpio_dev, 23, GPIO_OUTPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.23.");
    }
    if(gpio_pin_configure(gpio_dev, 24, GPIO_OUTPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.24.");
    }
    if(gpio_pin_configure(gpio_dev, 25, GPIO_OUTPUT | GPIO_ACTIVE_LOW) < 0){
        LOG_ERR("Could not configure P0.25.");
    }
    if(gpio_pin_configure(gpio_dev, 28, GPIO_OUTPUT | GPIO_ACTIVE_HIGH) < 0){
        LOG_ERR("Could not configure P0.28.");
    }
for(i = 0;i <MAX_OUTPUT;i++){
        digio_Out(i, 0);
    }
    gpio_pin_set(gpio_dev, 24, 0);            // PWR_INP passive
    gpio_pin_set(gpio_dev, 23, 0);            // EN_PWR passive
    gpio_pin_set(gpio_dev, 28, 0);            // EN_NTC passive
    
}

void digio_Out(uint8_t ch, uint8_t state) 
{

    if( ch < CS_PWR1){
        spi_Out(digio_RemapOut(ch), state);    
        
    }
    else{
        switch (ch) {
            case CS_PWR1:
                if (state) {
                    gpio_pin_set(gpio_dev, 25, 1);
                } else {
                    gpio_pin_set(gpio_dev, 25, 0);
                }
                break;

            case CS_PWR2:
                if (state) {
                     gpio_pin_set(gpio_dev, 18, 1);
                } else {
                     gpio_pin_set(gpio_dev, 18, 0);
                }
                break;

            case EN_PWR:
                if (state) {
                    gpio_pin_set(gpio_dev, 23, 1);
                } else {
                    gpio_pin_set(gpio_dev, 23, 0);
                }
                break;
    
            case PWR_INP:
                if (state) {
                    gpio_pin_set(gpio_dev, 24, 1);                   
                } else {
                    gpio_pin_set(gpio_dev, 24, 0);
                }
                break;
           

            default:
                break;
        }
    }
}

bool digio_In(uint8_t ch) 
{
    bool result;
   
    
    gpio_pin_set(gpio_dev, 24, 1);            // PWR_INP
    k_sleep(K_MSEC(1));
    switch (ch) {    
        case DOOR1_CLOSED:            
            result = gpio_pin_get(gpio_dev, 11);
            break;

        case DOOR2_CLOSED:            
            result = gpio_pin_get(gpio_dev, 12);
            break;

        case DOOR3_CLOSED:            
            result =gpio_pin_get(gpio_dev, 13);
            break;

        case DOOR4_CLOSED:            
            result =gpio_pin_get(gpio_dev, 14);
            break;

        case DOOR5_CLOSED:            
            result = gpio_pin_get(gpio_dev, 15);
            break;

        case DOOR6_CLOSED:            
            result =gpio_pin_get(gpio_dev, 8);
            break;

        case DOOR7_CLOSED:            
            result = gpio_pin_get(gpio_dev, 9);
            break;

        case DOOR8_CLOSED:            
            result = gpio_pin_get(gpio_dev, 10);
            break;

        case DOOR9_CLOSED:            
           result = gpio_pin_get(gpio_dev, 5);
            break;

        case DOOR10_CLOSED:            
            result = gpio_pin_get(gpio_dev, 6);
            break;

        case DOOR11_CLOSED:            
            result = gpio_pin_get(gpio_dev, 7);
            break;

        case DOOR12_CLOSED:            
            result = gpio_pin_get(gpio_dev, 0);
            break;

        case DOOR13_CLOSED:            
            result = gpio_pin_get(gpio_dev, 1);
            break;
          
        case DOOR14_CLOSED:            
            result = gpio_pin_get(gpio_dev, 2);
            break;

        case DOOR15_CLOSED:            
            result = gpio_pin_get(gpio_dev, 3);
            break;
       
         case DOOR16_CLOSED:            
            result = gpio_pin_get(gpio_dev, 4);
            break;   
                

        default:
            result = false;
            break;
    }
    gpio_pin_set(gpio_dev, 24, 0);            // PWR_INP passive
   
    return (result);
}	

bool    digio_State(uint8_t ch)
{
    return(out_state[ch]);
}

void  qlocx_digio_PwrOn(bool state)
{
    if(state){
         gpio_pin_set(gpio_dev, 23, 1);            // EN_PWR
    }
    else{
         gpio_pin_set(gpio_dev, 23, 0);            // EN_PWR passive
    }

}

void  qlocx_digio_NTCPwrOn(bool state)
{
    if(state){
         gpio_pin_set(gpio_dev, 28, 1);            // EN_NTC
    }
    else{
         gpio_pin_set(gpio_dev, 28, 0);            // EN_NTC passive
    }

}

void  qlocx_digio_InpPwrOn(bool state)
{
    if(state){
         gpio_pin_set(gpio_dev, 24, 1);            // PWR_INP
    }
    else{
         gpio_pin_set(gpio_dev, 24, 0);            // PWR_INP passive
    }

}

uint8_t qlocx_digio_GetStatus(uint8_t index)
{
uint8_t result;

    switch(index){
        case 0:
          result = open_fault_pwr1;
          break;

        case 1:
          result = short_fault_pwr1;
          break;
        
        case 2:
          result = open_fault_pwr2;
          break;

        case 3:
          result = short_fault_pwr2;
          break;
            
        default:
          result = 0xAA;
          break;
    }
    return(result);           
}