/***************************************************************************
 * qlocx_door_obj.c 				                                                
 *											
 *  
 *  
 *                                                                                      
 * Functional level: 1									
 *											
 * Prepared: Dirk Handzic							
 *																					
 * 											
 * Rev	Date	Author	Ext int. (Y/N) 	Reason/description
 * 0.1  201106  DH	Y        	Initial version
 *
 *  
 * Door lock LWM2M configuration 										
 ***************************************************************************/
#include        <zephyr.h>
#include        <stdlib.h>
#include        <nrf9160.h>
#include        <drivers/spi.h>
#include        <drivers/gpio.h>
#include        <zephyr.h>
#include        <net/lwm2m.h>
#include        <stdint.h>
#include        <init.h>
#include	"qlocx_digio.h"
#include        <logging/log.h>
#include        "lwm2m_object.h"
#include        "lwm2m_engine.h"

//*************************** Definitions ***********************************

#define LOG_MODULE_NAME net_lwm2m_obj_qlocx_door
#define LOG_LEVEL CONFIG_LWM2M_LOG_LEVEL

LOG_MODULE_REGISTER(LOG_MODULE_NAME);

#define LWM2M_OBJECT_DOOR_ID                    10351

/* resource IDs */
#define DOOR_NAME_ID                            1
#define DOOR_STATUS_ID                          50
#define DOOR_CONTROL_ID                         100

#define DOOR_MAX_ID				3

#define MAX_INSTANCE_COUNT                      16

#define DOOR_STRING_SHORT                       20
#define DOOR_STRING_LONG                        64

/*
 * Calculate resource instances as follows:
 * start with DOOR_MAX_ID
 */
#define RESOURCE_INSTANCE_COUNT	(DOOR_MAX_ID)

/* resource state variables */
static bool open_closed_value[MAX_INSTANCE_COUNT];
static char control[MAX_INSTANCE_COUNT][DOOR_STRING_SHORT];
static char name[MAX_INSTANCE_COUNT][DOOR_STRING_LONG];

static struct lwm2m_engine_obj door;
static struct lwm2m_engine_obj_field fields[] = {
	OBJ_FIELD_DATA(DOOR_NAME_ID, RW, STRING),
	OBJ_FIELD_DATA(DOOR_STATUS_ID, R, BOOL),
	OBJ_FIELD_DATA(DOOR_CONTROL_ID, W, STRING),	
};

static struct lwm2m_engine_obj_inst inst[MAX_INSTANCE_COUNT];
static struct lwm2m_engine_res res[MAX_INSTANCE_COUNT][DOOR_MAX_ID];
static struct lwm2m_engine_res_inst res_inst[MAX_INSTANCE_COUNT][RESOURCE_INSTANCE_COUNT];

static struct lwm2m_engine_obj_inst *door_create(uint16_t obj_inst_id)
{
	int index, avail = -1, i = 0, j = 0;

	/* Check that there is no other instance with this ID */
	for (index = 0; index < MAX_INSTANCE_COUNT; index++) {
		if (inst[index].obj && inst[index].obj_inst_id == obj_inst_id) {
			LOG_ERR("Can not create instance - "
				"already existing: %u", obj_inst_id);
			return NULL;
		}

		/* Save first available slot index */
		if (avail < 0 && !inst[index].obj) {
			avail = index;
		}
	}

	if (avail < 0) {
		LOG_ERR("Can not create instance - no more room: %u",
			obj_inst_id);
		return NULL;
	}

	/* Set default values */
	open_closed_value[avail] = false;
	control[avail][0] = '\0';
	name[avail][0] = '\0';

	(void)memset(res[avail], 0,
		     sizeof(res[avail][0]) * ARRAY_SIZE(res[avail]));
	init_res_instance(res_inst[avail], ARRAY_SIZE(res_inst[avail]));

	/* initialize instance resource data */
	INIT_OBJ_RES_DATA(DOOR_STATUS_ID, res[avail], i, res_inst[avail], j,
			  &open_closed_value[avail], sizeof(*open_closed_value));
	
	INIT_OBJ_RES_DATA(DOOR_NAME_ID, res[avail], i,
			  res_inst[avail], j,
			  name[avail], DOOR_STRING_LONG);

	INIT_OBJ_RES_DATA(DOOR_CONTROL_ID, res[avail], i,
			  res_inst[avail], j,
			  control[avail], DOOR_STRING_SHORT);
	

	inst[avail].resources = res[avail];
	inst[avail].resource_count = i;

	LOG_DBG("Create IPSO Qlocx Door Control instance: %d", obj_inst_id);

	return &inst[avail];
}

static int ipso_door_init(const struct device *dev)
{	
	struct lwm2m_engine_obj_inst *obj_inst = NULL;

	door.obj_id = LWM2M_OBJECT_DOOR_ID;
	door.fields = fields;
	door.field_count = ARRAY_SIZE(fields);
	door.max_instance_count = ARRAY_SIZE(inst);
	door.create_cb = door_create;
	lwm2m_register_obj(&door);

	return 0;
}

SYS_INIT(ipso_door_init, APPLICATION, CONFIG_KERNEL_INIT_PRIORITY_DEFAULT);