/***************************************************************************
 * qlocx_volt_obj.c 				                                                
 *											
 *  
 *  
 *                                                                                      
 * Functional level: 1									
 *											
 * Prepared: Dirk Handzic							
 *																					
 * 											
 * Rev	Date	Author	Ext int. (Y/N) 	Reason/description
 * 0.2  201215  DH      N               Updated comments
 * 0.1  201106  DH	Y        	Initial version
 *
 *  
 * Board voltage LWM2M configuration 										
 ***************************************************************************/
#include        <zephyr.h>
#include        <stdlib.h>
#include        <nrf9160.h>
#include        <drivers/spi.h>
#include        <drivers/gpio.h>
#include        <zephyr.h>
#include        <net/lwm2m.h>
#include        <stdint.h>
#include        <init.h>
#include	"qlocx_digio.h"
#include        <logging/log.h>
#include        "lwm2m_object.h"
#include        "lwm2m_engine.h"

//*************************** Definitions ***********************************

#define LOG_MODULE_NAME net_lwm2m_obj_qlocx_volt
#define LOG_LEVEL CONFIG_LWM2M_LOG_LEVEL

LOG_MODULE_REGISTER(LOG_MODULE_NAME);

#define LWM2M_OBJECT_VOLT_ID                    3316

#ifdef CONFIG_LWM2M_IPSO_VOLT_SENSOR_TIMESTAMP
#define ADD_TIMESTAMPS 1
#else
#define ADD_TIMESTAMPS 0
#endif

/* Server resource IDs */
#define VOLT_SENSOR_VALUE_ID			5700
#define VOLT_UNITS_ID				5701
#if ADD_TIMESTAMPS
#define VOLT_TIMESTAMP_ID			5518

#define VOLT_MAX_ID		3
#else  /* !ADD_TIMESTAMPS */
#define VOLT_MAX_ID		2
#endif

#define MAX_INSTANCE_COUNT	1

#define VOLT_STRING_SHORT	8

/*
 * Calculate resource instances as follows:
 * start with VOLT_MAX_ID
 * subtract EXEC resources (1)
 */
#define RESOURCE_INSTANCE_COUNT	(VOLT_MAX_ID)

/* resource state variables */
static float32_value_t sensor_value[MAX_INSTANCE_COUNT];
static char units[MAX_INSTANCE_COUNT][VOLT_STRING_SHORT];


static struct lwm2m_engine_obj volt_sensor;
static struct lwm2m_engine_obj_field fields[] = {
	OBJ_FIELD_DATA(VOLT_SENSOR_VALUE_ID, R, FLOAT32),
	OBJ_FIELD_DATA(VOLT_UNITS_ID, R_OPT, STRING),
#if ADD_TIMESTAMPS
	OBJ_FIELD_DATA(VOLT_TIMESTAMP_ID, RW_OPT, TIME),
#endif
};

static struct lwm2m_engine_obj_inst inst[MAX_INSTANCE_COUNT];
static struct lwm2m_engine_res res[MAX_INSTANCE_COUNT][VOLT_MAX_ID];
static struct lwm2m_engine_res_inst res_inst[MAX_INSTANCE_COUNT][RESOURCE_INSTANCE_COUNT];


static int sensor_value_write_cb(uint16_t obj_inst_id,
				 uint16_t res_id, uint16_t res_inst_id,
				 uint8_t *data, uint16_t data_len,
				 bool last_block, size_t total_size)
{
	

	return 0;
}

static struct lwm2m_engine_obj_inst *volt_sensor_create(uint16_t obj_inst_id)
{
	int index, i = 0, j = 0;

	/* Check that there is no other instance with this ID */
	for (index = 0; index < MAX_INSTANCE_COUNT; index++) {
		if (inst[index].obj && inst[index].obj_inst_id == obj_inst_id) {
			LOG_ERR("Can not create instance - "
				"already existing: %u", obj_inst_id);
			return NULL;
		}
	}

	for (index = 0; index < MAX_INSTANCE_COUNT; index++) {
		if (!inst[index].obj) {
			break;
		}
	}

	if (index >= MAX_INSTANCE_COUNT) {
		LOG_ERR("Can not create instance - no more room: %u",
			obj_inst_id);
		return NULL;
	}

	/* Set default values */
	sensor_value[index].val1 = 0;
	sensor_value[index].val2 = 0;
	units[index][0] = 'V';
        units[index][1] = '\0';
	

	(void)memset(res[index], 0,
		     sizeof(res[index][0]) * ARRAY_SIZE(res[index]));
	init_res_instance(res_inst[index], ARRAY_SIZE(res_inst[index]));

	/* initialize instance resource data */
	INIT_OBJ_RES(VOLT_SENSOR_VALUE_ID, res[index], i,
		     res_inst[index], j, 1, true,
		     &sensor_value[index], sizeof(*sensor_value),
		     NULL, NULL, sensor_value_write_cb, NULL);
	INIT_OBJ_RES_DATA(VOLT_UNITS_ID, res[index], i, res_inst[index], j,
			  units[index], VOLT_STRING_SHORT);
	
#if ADD_TIMESTAMPS
	INIT_OBJ_RES_OPTDATA(VOLT_TIMESTAMP_ID, res[index], i,
			     res_inst[index], j);
#endif

	inst[index].resources = res[index];
	inst[index].resource_count = i;
	LOG_DBG("Create IPSO Voltage Sensor instance: %d", obj_inst_id);
	return &inst[index];
}

static int ipso_volt_sensor_init(const struct device *dev)
{
	volt_sensor.obj_id = LWM2M_OBJECT_VOLT_ID;
	volt_sensor.fields = fields;
	volt_sensor.field_count = ARRAY_SIZE(fields);
	volt_sensor.max_instance_count = MAX_INSTANCE_COUNT;
	volt_sensor.create_cb = volt_sensor_create;
	lwm2m_register_obj(&volt_sensor);

	return 0;
}

SYS_INIT(ipso_volt_sensor_init, APPLICATION,
	 CONFIG_KERNEL_INIT_PRIORITY_DEFAULT);
